/**
 * Tecnología del ocio
 * Bloque 1, unidad didáctica 8
 * Práctica 2
 */

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <cwiid.h>

cwiid_err_t error; //Función para errores

//Función para errores
void error(cwiid_wiimote_t *wiimote, const char *s, va_list ap) {
	if (wiimote != NULL) {printf("Error en el dispositivo %d: ", cwiid_get_id(wiimote));}
	vprintf(s, ap);
	printf("\n");
}

int main(int argc, char *argv[]) {

	cwiid_wiimote_t* wiimote; //Gestor del wiimote
	bdaddr_t bdaddr; //Dirección Bluetooth del wiimote
	int mesg_count; //Número de mensajes obtenidos
	union cwiid_mesg *mesg; //Array de mensajes obtenido
	struct timespec t; //Instante de obtención del mensaje
	char tmp[256]; //Temporal para mostrar la dirección Bluetooth
	int i; //Índice del bucle que recorre los mensajes
	int salir = 0; //Control del bucle principal
	char nombres_btn[6][10] = {"A", "B", "Arriba", "Abajo", "Derecha", "Izquierda"}; //Nombre de los botones
	int id_btn[6] = {8, 4, 2048, 1024, 512, 256}; //Identificadores de los botones
	int sorteo = -1; //Número de botón sorteado, -1 indica que no existe (es el primer sorteo o se acertó el último)
	struct timespec tinicio; //Instante en que se anunció qué botón pulsar

	cwiid_set_err(error); //Asigno la función de error

	printf("Poner el wiimote en modo de descubrimiento (pulsar 1+2) y pulsar enter...\n");
	getchar();
	printf("Buscando...\n");
	bdaddr = *BDADDR_ANY; //Se aceptará la conexión de cualquier wiimote
	if (!(wiimote = cwiid_connect(&bdaddr, CWIID_FLAG_MESG_IFC))) { //Conexión y habilitación de envío de mensajes
		printf("No se pudo detectar wiimote\n");
	}
	else {
		//Conexión establecida
		ba2str(&bdaddr, tmp); //Traducción a cadena de caracteres de la dirección Bluetooth
		printf("Dirección Bluetooth del wiimote %i: %s\n", cwiid_get_id(wiimote), tmp);
		if (cwiid_set_rpt_mode(wiimote, CWIID_RPT_BTN)) { //Sólo pedir mensajes relacionados con los botones
			printf("Error al establecer el modo de informe.\n");
			salir = 1; //Evitar que entre en el bucle
		}
		printf("\nPara salir pulsa el botón 1 del Wiimote\n\n");
		srand(time(NULL)); //Inicializar la semilla aleatoria
		while (!salir) {
			if (sorteo == -1) {
				sorteo = rand() % 6;
				printf("----- ATENCIÓN -----\n");
				printf("-- Pulsa el botón --\n");
				printf("----  %9s ----\n", nombres_btn[sorteo]);
				clock_gettime(CLOCK_REALTIME, &tinicio);
			}
			//la ejecución se bloqueará aquí hasta que haya un mensaje
			if (cwiid_get_mesg(wiimote, &mesg_count, &mesg, &t) != 0) {
				printf("Error al obtener mensajes del wiimote\n");
			}
			else {
				for (i = 0; i < mesg_count; i++) {
					if (mesg[i].type == CWIID_MESG_BTN) {
						if (mesg[i].btn_mesg.buttons == 2) { //Botón 1 pulsado, salir
							salir = 1;
						}
						else if (mesg[i].btn_mesg.buttons == id_btn[sorteo]) { //Botón correcto pulsado
							printf("\nCORRECTO\n");
							printf("Pulsaste la tecla en %f segundos\n\n", (t.tv_sec-tinicio.tv_sec) + (t.tv_nsec-tinicio.tv_nsec)/1E9);
							sorteo = -1;
						}
						else if (mesg[i].btn_mesg.buttons != 0) { //No hay ningún botón pulsado, ignorar
							printf("\nINCORRECTO\n");
						}
					}
				}
			}
		}
		if (cwiid_disconnect(wiimote)) { //Desconexión del wiimote
			printf("Error al desconectar el wiimote\n");
		}
	}
	return 0;
}
